setwd("C:/Users/Anne/Dropbox/PhD/Anne & Shawn/Beta MS/Data for Beta MS")

####      Spatial Variation in Multi-Trophic Antagonistic Networks
####
####  ------------------------------------------------------------
####
####  Anne M. McLeod, Cindy Chu, and Shawn J. Leroux
####
####
####  Code to replicate analyses carried out and create figures
####  -------------------------------------------
####
####  CONTENTS:
####  ---------
####  1. Setting up the Metaweb
####  2. Creating the lake-specific probability matrix for each ecological structuring mechanism.
####     a) Abundance and Morphology
####     b) Phylogenetic distances
####     c) Thermal optima
####     d) Competition
####     e) Calculating all permutations of Abundance, Morphology, Phylogenetic Distance, Thermal Optima, and Competition
####     f) Normalizing the lake-specific probability matrices and adding the null model
####     g) Adding the observed interactions
####     h) Removing rare interactions (see Appendix S2)
####  3. Selecting interactions based on the ecological structuring mechanisms and comparing to the observed interactions.
####  4. Calculating interaction turnover between lakes and creating Beta boxplots.
####     a) Calculating interaction turnover
####     b) Creating the beta boxplots
####  5. Evaluating environmental drivers of interaction turnover.  
####  6. Determining species dissimilarity and creating dissimilarity violin plots.     
    

#### Packages

#Compiling functions
#install.packages("compiler")
library(compiler)

#Data Management
#install.packages("data.table")
library(data.table)

#adonis analysis
#install.packages("vegan")
library(vegan) 

#Create Phylogeny
#install.packages("ape")
library(ape) 

#Calculate Phylogenetic Distance
#install.packages("phangorn")
library(phangorn) 

#Create figures
#install.packages("ggplot2")
library(ggplot2)

#### Functions

## Part 2 - Creating the lake-specific probability matrix for each ecological structuring mechanism.

# Function that calculates the proportion of overlap between two thermal tolerances using the two thermal optima,
# and the two standard deviations
overlap_f <- function(x, Topt1, Topt2, sd1, sd2) {
  f1 <- dnorm(x, mean=Topt1, sd=sd1)
  f2 <- dnorm(x, mean=Topt2, sd=sd2)
  pmin(f1, f2)
}

# Function calculates the total area under the thermal optima curve using a thermal optima and a standard deviation
total_f <- function(x, Topt, sd) {
  dnorm(x, Topt, sd)
}

# Function which adds the probability of interaction due to exploitative competition to the probability matrix.
# New_M is the new matrix which includes the competition probabilites, Old_M is the probability matrix
#This function returns NewM, the new matrix with the competition probabilities added
Competition <- function(old_M, New_M){
  for (i in 1:length(unique(old_M$Lake_ID))){
    temp_lake <- subset(old_M, old_M$Lake_ID == unique(old_M$Lake_ID)[i])
    for (j in 1:length(unique(temp_lake$Prey))){
      Prey <- subset(temp_lake, temp_lake$Prey == unique(temp_lake$Prey)[j])
      total_pred <- sum(Prey$Interaction)
      Prey$C_p <- ifelse(total_pred > 0, 1/total_pred, 0)
      New_M <- rbind(New_M, Prey)
    }
  }
  return(New_M)
}

Competition_Compiled <- cmpfun(Competition)

## Part 3 - Selecting interactions based on the ecological structuring mechanisms and comparing to observed interactions

# Reshuffle function which selects interactions between specied based on their probability of occurring.
# Here ID is the unique ID for each pred-prey interaction in each lake, p is the probability of the interactions 
# occurring, and Int_Obs is the number of interactions actually observed in that lake. It returns a list of unique
# IDs
Reshuffle <- function(ID, p, Int_Obs) {
  resample <- sample(ID, size = Int_Obs, replace = FALSE, prob = p)
  Pred_Int <- as.integer(ID %in% resample)
  return(Pred_Int)
}

Reshuffle_Compile <-cmpfun(Reshuffle)

# Function which calculates the True Skill Statistic where ProbM is the matrix with interactions and probabilities, NewM is the
# New matrix which gets filled with u, v, w, and x (where where u are the interactions which were both predicted 
# and observed to occur, v are the interactions which were predicted to occur, but observed absent, w are the interactions 
# which were predicted to be absent, but observed to occur, and x are the interactions which were both predicted 
# and observed to be absent), the TSS, and the Sensitivity score (Sens). Var_Name is the ecological mechanism name (e.g. A, G, T etc.)
# This function returns the New Matrix with the true skill statistic components.
TSS_Calc <- function(ProbM, NewM, Var_Name){
  for (i in 1:5000){
    print(i)
    names(ProbM)[names(ProbM) == Var_Name] <- "prob"
    temp <- ProbM[, .(Predator = Predator, Prey = Prey, Int_Obs = Int_Obs, Predicted_Int = Reshuffle_Compile(ID, prob, sum(Int_Obs))), by = Lake_ID]
    temp2 <- temp[, .(u = sum(Int_Obs == Predicted_Int & Predicted_Int == 1), v = sum(Int_Obs == 0 & Predicted_Int == 1), w = sum(Int_Obs == 1 & Predicted_Int == 0), x = sum(Int_Obs == Predicted_Int & Predicted_Int == 0)), by = Lake_ID]
    temp2$TSS <- (temp2$u*temp2$x - temp2$v*temp2$w)/((temp2$u+temp2$w)*(temp2$v + temp2$x))
    temp2$Sens <- temp2$u/(temp2$u+temp2$w)
    temp2$Iter <- i
    NewM <- rbind(NewM, as.data.frame(temp2))
  }
  return(NewM)
}

TSS_Calc_Compiled <- cmpfun(TSS_Calc)

## Part 4 - Calculating interaction turnover between lakes

# Function which calculates the different components for the beta calculations where FinalM is the new matrix which gets 
# filled with the beta components #including the two lakes being compared, the number of shared interactions (a), the 
# number of interactions unique to site 1 (b), the number #of interactions unique to site 2 (c), the number of interactions 
# which are rewired between the two sites (Rewired), the number of interactions #that change due to turnover in top, basal, 
# intermediate, and both novel species, and finally, the difference in species diversity between the #two sites.  IntM is 
# the observed interaction matrix. This function returns FinalM, the new matrix with the different beta components.
Beta_Calc <- function(FinalM, IntM){
  index <- 0
  for (i in 1:(length(unique(IntM$Lake_ID))-1)){
    #print(i)
    L1 <- as.character(unique(IntM$Lake_ID)[i])
    L1_sub <- subset(IntM, IntM$Lake_ID == L1)
    Species_A <- L1_sub$Prey
    for (j in (i+1):length(unique(IntM$Lake_ID))){
      index = index + 1
      L2 <- as.character(unique(IntM$Lake_ID)[j])
      L2_sub <- subset(IntM, IntM$Lake_ID == L2)
      Species_B <- L2_sub$Prey
      Prey <- union(subset(L1_sub, L1_sub$Int_Obs>0)$Prey, subset(L2_sub, L2_sub$Int_Obs>0)$Prey)
      
      L1_sub$Both_P <- ifelse(L1_sub$Predator %in% Species_B & L1_sub$Prey %in% Species_B, 1, 0)
      L1_sub$Both_A <- ifelse(!(L1_sub$Predator %in% Species_B) & !(L1_sub$Prey %in% Species_B), 1, 0)
      L1_sub$Pred_A <- ifelse(L1_sub$Prey %in% Species_B & !(L1_sub$Predator %in% Species_B) & !(L1_sub$Predator %in% Prey), 1, 0)
      L1_sub$Intermediate <- ifelse(L1_sub$Prey %in% Species_B & !(L1_sub$Predator %in% Species_B) & (L1_sub$Predator %in% Prey), 1, 0)
      L1_sub$Prey_A <- ifelse(L1_sub$Predator %in% Species_B & !(L1_sub$Prey %in% Species_B) & L1_sub$Intermediate == 0, 1, 0)
      
      
      L2_sub$Both_P <- ifelse(L2_sub$Predator %in% Species_A & L2_sub$Prey %in% Species_A, 1, 0)
      L2_sub$Both_A <- ifelse(!(L2_sub$Predator %in% Species_A) & !(L2_sub$Prey %in% Species_A), 1, 0)
      L2_sub$Pred_A <- ifelse(L2_sub$Prey %in% Species_A & !(L2_sub$Predator %in% Species_A) & !(L2_sub$Predator %in% Prey), 1, 0)
      L2_sub$Intermediate <- ifelse(L2_sub$Prey %in% Species_A & !(L2_sub$Predator %in% Species_A) & (L2_sub$Predator %in% Prey), 1, 0)
      L2_sub$Prey_A <- ifelse(L2_sub$Predator %in% Species_A & !(L2_sub$Prey %in% Species_A) & L2_sub$Intermediate == 0, 1, 0)
      
      L1_L2 <- rbind(L1_sub[which(L1_sub$Int_Obs >0),], L2_sub[which(L2_sub$Int_Obs > 0),])
      
      a <- nrow(L1_L2) - nrow(unique(L1_L2[,c(2:4)]))
      b <- nrow(subset(L1_sub, L1_sub$Int_Obs > 0)) - a
      c <- nrow(subset(L2_sub, L2_sub$Int_Obs > 0)) - a
      Rewired <- sum(L1_L2$Both_P) - 2*a
      Novel <- sum(L1_L2$Both_A)
      Basal <- sum(L1_L2$Prey_A)
      Top <- sum(L1_L2$Pred_A)
      Intermediate <- nrow(L1_L2) - 2*a - Rewired - Novel - Basal - Top
      Num_Diff_Species <- length(union(Species_A, Species_B)) - length(intersect(Species_A, Species_B))
      FinalM[index, ] <- c(L1, L2, a, b, c, Rewired, Novel, Basal, Top, Intermediate, Num_Diff_Species)
    }
  }
  return(FinalM)
}

Beta_Calc_Compile <- cmpfun(Beta_Calc)

# Function which takes the matrix (M1) with the lake by lake comparisons and adds columns for each of the 
# betas. This function returns this matrix (M1).
B_Calcs <- function(M1) {
  M1$Brw <- ifelse(M1$Rewired == 0 & M1$a == 0, 0, (M1$Rewired)/(2*M1$a + M1$b + M1$c))
  M1$Bint <- (M1$a + M1$b + M1$c) / ((2*M1$a + M1$b + M1$c)/2) - 1
  M1$Bst <- M1$Bint - M1$Brw
  M1$Bb <- (M1$Basal)/(2*M1$a + M1$b + M1$c)
  M1$Bt <- (M1$Top)/(2*M1$a + M1$b + M1$c)
  M1$Bintermediate <- (M1$Intermediate)/(2*M1$a + M1$b + M1$c)
  M1$Bnovel <- M1$Bst - M1$Bb - M1$Bt - M1$Bintermediate
  return(M1)
}
B_Calcs_compiled <- cmpfun(B_Calcs)

# function for computing mean, DS, max and min values. This is used for creating Figure 4 (box plot of beta values against 
# type of turnover).
min.mean.sd.max <- function(x) {
  r <- c(quantile(x,0.05), quantile(x, 0.25), mean(x), quantile(x, 0.75), quantile(x, 0.95))
  names(r) <- c("ymin", "lower", "middle", "upper", "ymax")
  r
}

## Part 5 - Determining species dissimilarity

# Function which takes the matrix with the position of each species in each lake and calculates
# dk, or species dissimilarity. It returns the value dk.
Dist_Calc <- function(Species_Pos){
  D <- 1/nrow(Species_Pos)
  e <- 0
  for (i in 1:(nrow(Species_Pos)-1)){
    for (j in (i+1):nrow(Species_Pos)){
      Fij <- 0
      for (k in 3:ncol(Species_Pos)){
        if (Species_Pos[[j,k]] == 0 && Species_Pos[[i,k]] == 0){
          Fij <- Fij
        }
        else {
          Fij <- Fij + min(Species_Pos[[j,k]], Species_Pos[[i,k]])
        }
      }
      eij <- 1-(Fij)
      e <- e + (eij)^2
    }
  }
  D <- D*e
  return(D)
}
Dist_Calc_Compiled <- cmpfun(Dist_Calc)

# Function takes the matrix with the position of each species in each lake and an empty matrix Dw, and 
# puts the value from the Dist_Calc function into the new matrix Dw. It returns Dw.
Dw_Calc <- function(Species_Pos, Dw){
  Species_List <- (unique(Species_Pos$Species))
  for (m in 1:length(Species_List)){
    Species_Sub <- subset(Species_Pos, Species_Pos$Species == Species_List[m])
    if (nrow(Species_Sub)>1){   #because for some species there is only one. e.g. brown bullhead.
      D <- Dist_Calc_Compiled(Species_Sub)
      Dw[1,as.character(Species_List[m])] <- D
    }
  }
  return(Dw)
}

Dw_Calc_Compiled <- cmpfun(Dw_Calc)



## Part 1 - Setting up the Metaweb
## -------------------------------

# Import the data
Metaweb <- read.csv("Metaweb.csv", header = TRUE)

# These data were collected by the Broadscale Monitoring Program, the raw data were compiled into a metaweb 
# from the stomach content information collected from 741 lakes across Ontario from 2008 - 2012.
#
# Citation: Sandstrom, S.., Rawson, S.M. & Lester, N. (2013). Manual 
#           of Instructions for Broad-scale Fish Community Monitoring; 
#           using North American (NA1) and Ontario Small Mesh (ON2) 
#           Gillnets. Ontario Ministry of Natural Resources, Peterborough, Ontario.
#
# Data details: 
#  [1] Predator: common name for the consumer in the interaction
#  [2] Prey: common name for the prey in the interaction
#  [3] Interaction: 1 or 0 depending if the interaction is observed or absent, respectively.

# Import the environmental data
Lake_Environment <- read.csv("Lake_Environment.csv", header = TRUE)

# These data were collected by the Broadscale Monitoring Program, and this file contains the environmental
# data for each of the 129 lakes used in the study.
#
# Citation: Sandstrom, S.., Rawson, S.M. & Lester, N. (2013). Manual 
#           of Instructions for Broad-scale Fish Community Monitoring; 
#           using North American (NA1) and Ontario Small Mesh (ON2) 
#           Gillnets. Ontario Ministry of Natural Resources, Peterborough, Ontario.
#
# Data details:
#   [1] Lake_ID: The unique Lake ID for each of the lakes used in the study
#   [2] Latitude_DD: The latitude of each of the lakes
#   [3] Longitude_DD: The longitude of each of the lakes
#   [3] Area_ha: The area of each lake in hectares
#   [4] Depmn_m: The mean depth of each lake in m
#   [5] TDS_mgL: Total Dissolved Solids in each lake in mg L-1
#   [6] pH: The pH of each lake
#   [7] TP: The total Phosphorus content of each lake in ??g L-1
#   [8] DD5_8110: The number of degree days above 5 C.
#   [9] Drainage: The drainage basin the lake belongs to

Final_Lakes <- unique(Lake_Environment$Lake_ID)

# Remove Cannibalism from metaweb because otherwise models overpredict its importance.
for (i in 1:nrow(Metaweb)){
  if (as.character(Metaweb$Predator[i]) == as.character(Metaweb$Prey[i])){
    Metaweb$Interaction[i] <- 0
  }
}



## Part 2 - Creating the lake-specific probability matrix for each ecological structure mechanism
## ----------------------------------------------------------------------------------------------

# a) Abundance and Morphology:

# Import file with lake-specific probability of interactions based on species abundance and morphology
Abundance_Morphology <- read.csv(file = "Abundance_Morphology_Final.csv", header = TRUE)

# This file contains lake-specific probabilities based on species abundance and morphology for interactions
# between co-occurring species.
#
# Data details:
#  [1] Lake_ID: The unique Lake ID for each of the lakes used in the study
#  [2] Predator: common name for the consumer in the interaction
#  [3] Prey: common name for the prey in the interaction
#  [4] A_p: The probability of interaction between predator and prey species based on species abundance
#  [5] M_p: The probability of interaction between predator and prey species based on species morphology


Abundance <- merge(Metaweb, Abundance_Morphology, by= c("Predator", "Prey"), all.y = TRUE)
# some NAs in the metaweb because prey fish were never observed in gut contents of the predators. So I set these as 0s since they
# are present, but not consumed
Abundance[is.na(Abundance)] <- 0
Abundance <- Abundance[,c(4, 1, 2, 3, 5, 6)]

# b) Phylogenetic Distances:

# Import fasta file with DNA-barcode sequence for freshwater fish in Ontario
fish_phylo <- read.phyDat("Fish_Phylo.fas", format = "fasta") 

# This file is a text-based DNA-barcode sequence information for freshwater 
# fish in Ontario. Sequences are aligned the same way as in Hubert et al. (2008)
# but the distances are calculated using the phangorn package.
#
# Citation: Hubert, N., Hanner, R., Holm, E., Mandrak, N.E., Taylor, E., 
#           Burridge, M., et al. (2008). Identifying Canadian freshwater 
#           fishes through DNA barcodes. PLoS One, 3, e2490.

# Import species list matching latin names with common names
spec_list <- read.csv("Species_List.csv", header = TRUE)

# This file contains all of the species found in the lakes along with their latin names.
#
# Data details:
#  [1] SpecName: The species' common name
#  [2] species: the corresponding latin name for the species.

fish_phylo.dist <- dist.ml(fish_phylo)     #calculates distance
fish_phylo_matrix <- as.matrix(fish_phylo.dist) #changes to a matrix from distance object
fish_phylo <- setDT(data.frame(fish_phylo_matrix), keep.rownames = TRUE)[] #makes first column the Species_A name

# Changing distances from a matrix to column form
fish_phylo <- data.frame(fish_phylo)
fish <- names(fish_phylo)
fishDF <- data.frame(character(), double(), character())
names(fishDF) <- c("Species_A", "Distance", "Species_B")
for (i in 2:ncol(fish_phylo)){
  print(i)
  temp <- fish_phylo[,c(1,i)]
  temp$Species_B <- fish[i]
  names(temp) <- names(fishDF)
  fishDF <- rbind(fishDF, temp)
}

# removing SpecID from the latin species name
fishDT <- setDT(fishDF)[, paste0("ID", 1:2) := tstrsplit(Species_A, "[|]")]
fishDT2 <- fishDT[, paste0("SpecB", 1:5) := tstrsplit(Species_B, "[.]")]

fishDT <- data.frame(fishDT2)[,c(2,5,8,9)]
fishDT$Species_B <- paste(fishDT$SpecB3, fishDT$SpecB4)
fishDT <- fishDT[,c(1,2,5)]
names(fishDT) <- c("Distance", "Species_A", "Species_B")

# multiple samples of same species, so need to take means of these values. 
fish_phylo <- setDT(fishDT)[, .(mean = mean(Distance)), by = c("Species_A", "Species_B")]
fish_phylo <- as.data.frame(fish_phylo)


spec_list <- spec_list[,c(1,2)]
names(spec_list) <- c("Sp_A", "Species_A")
fish2 <- merge(fish_phylo, spec_list, by = "Species_A")
names(spec_list) <- c("Sp_B", "Species_B")
fish3 <- merge(fish2, spec_list, by = "Species_B")

# Don't have distances for Splake, so take the average of Brook trout and Lake trout distances
Brook_Lake <- subset(fish3, fish3$Sp_A == "Brook Trout" | fish3$Sp_A == "Lake Trout")
Brook_Lake <- setDT(Brook_Lake)[, .(mean = mean(mean)), by = "Species_B"]
Brook_Lake <- as.data.frame(Brook_Lake)
Brook_Lake$Species_A <- "Splake"
Brook_Lake <- Brook_Lake[,c(3,1,2)]
fish3 <- fish3[,c(4,5,3)]
names(fish3) <- names(Brook_Lake)
fish_final <- rbind(fish3, Brook_Lake)

# Here 0s mean most similar, while 1s mean least. But want the opposite.
fish_final$mean_dis <- 1- fish_final$mean
fish_final <- fish_final[,c(1:2,4)]
names(fish_final) <- c("Predator", "Prey", "P_p")

A_M_P <- merge(Abundance, fish_final, by = c("Predator", "Prey"))

# c) Thermal Optima

# Import Thermal Optima for each fish species
Thermal <- read.csv("ThermalOptima.csv", header = TRUE)

# This file contains thermal optima for each species found in the lakes. For details on how the thermal
# optima were determined see Appendix S3.
#
# Data details:
#  [1] SpecName: The species' common name
#  [2] Topt: the thermal optima for the species in C.


# Import temperature profile for each lake.
Temperature <- read.csv("TemperatureProfiles.csv", header = TRUE)

# This file contains lake-specific temperature profiles collected during the Broadscale Monitoring Program 2008-2012.
#
# Data details:
#  [1] Lake_ID: The unique Lake ID for each of the lakes used in the study
#  [2] Depth.m.: The depth at which the temperature was measured (in m).
#  [3] Temp: The temperature at each depth (in C).


Thermal_Matrix <- data.frame(character(), character(), character(), double())
names(Thermal_Matrix) <- c("Lake", "Pred", "Prey", "p")

for (i in 1:length(unique(A_M_P$Lake_ID))){
  print(i)
  L1 <- unique(A_M_P$Lake_ID)[i]
  tempLake <- subset(A_M_P, A_M_P$Lake_ID == L1)
  if (nrow(Temperature[which(as.character(Temperature$Lake_ID) == as.character(L1)),]) > 1) { #A temp profile wasn't collected for every lake. This pares down the matrix to being only those lakes which have Temp Profiles
    minT <- min(Temperature[which(as.character(Temperature$Lake_ID) == as.character(L1)),]$Temp) #find the min temperature from the profile
    maxT <- max(Temperature[which(as.character(Temperature$Lake_ID) == as.character(L1)),]$Temp) #find the max temp from the profile for that lake
    
    for (j in 1:length(unique(tempLake$Predator))) {
      tempPred <- subset(tempLake, tempLake$Predator == unique(tempLake$Predator)[j])
      Topt1 = subset(Thermal, as.character(Thermal$SpecName) == as.character(unique(tempLake$Predator)[j]))$Topt
      sd1 = 2.02 #Standard deviation is 2.02C (Magnusson et al. 1979)
      sd2 = 2.02
      for (k in 1:length(unique(tempPred$Prey))) {
        Topt2 = subset(Thermal, as.character(Thermal$SpecName) == as.character(unique(tempPred$Prey)[k]))$Topt
        if  (Topt2 == 0 & Topt1 != 0){
          p12 = 1
        } else if (Topt1 != 0 & Topt2 != 0) {
          overlap <- integrate(overlap_f, 4, 25, Topt1, Topt2, sd1, sd2)
          Total1 <- integrate(total_f, 4, 25, Topt1, sd1)
          Total2 <- integrate(total_f, 4, 25, Topt2, sd2)
          if (Total1[1] == 0 | Total2[1] == 0) { #So that there is never a division by 0 error
            p12 = 0
          } else {
            p12 = as.numeric(overlap[1])/as.numeric(Total1[1])
          }
        }
        tempDF <- data.frame(unique(A_M_P$Lake_ID)[i], unique(tempLake$Predator)[j], unique(tempPred$Prey)[k], p12)
        names(tempDF) <- c("Lake_ID", "Predator", "Prey", "T_p")
        Thermal_Matrix <- rbind(Thermal_Matrix, tempDF)
      }
    }
  }
}


All_Prob <- merge(A_M_P, Thermal_Matrix, by = c("Lake_ID", "Predator", "Prey"))

# d) Competition

tempComp <- data.frame(character(), character(), character(), double(), double(), double(), double(), double(), double())
names(tempComp) <- c(names(All_Prob), "C_p")
All_Prob <- Competition_Compiled(All_Prob, tempComp)

# e) Calculating all permutations of Abundance, Morphology, Phylogenetic Distance, Thermal Optima, and Competition


All_Prob$A <- All_Prob$A_p*All_Prob$Interaction
All_Prob$M <- All_Prob$M_p*All_Prob$Interaction
All_Prob$P <- All_Prob$P_p*All_Prob$Interaction
All_Prob$Therm <- All_Prob$T_p*All_Prob$Interaction
All_Prob$AM <- All_Prob$A*All_Prob$M
All_Prob$AP <- All_Prob$A*All_Prob$P
All_Prob$AT <- All_Prob$A*All_Prob$Therm
All_Prob$MP <- All_Prob$M*All_Prob$P
All_Prob$MT <- All_Prob$M*All_Prob$Therm
All_Prob$PT <- All_Prob$P*All_Prob$Therm
All_Prob$AMP <- All_Prob$AM*All_Prob$P
All_Prob$AMT <- All_Prob$AM*All_Prob$Therm
All_Prob$APT <- All_Prob$AP*All_Prob$Therm
All_Prob$MPT <- All_Prob$MP*All_Prob$Therm
All_Prob$AMPT <- All_Prob$AMT*All_Prob$Therm

All_Prob$C <- All_Prob$C_p*All_Prob$Interaction
All_Prob$CA <- All_Prob$C*All_Prob$A
All_Prob$CM <- All_Prob$C*All_Prob$M
All_Prob$CP <- All_Prob$C*All_Prob$P
All_Prob$CT <- All_Prob$C*All_Prob$Therm
All_Prob$CAM <- All_Prob$CA*All_Prob$M
All_Prob$CAP <- All_Prob$CA*All_Prob$P
All_Prob$CAT <- All_Prob$CA*All_Prob$Therm
All_Prob$CMP <- All_Prob$CM*All_Prob$P
All_Prob$CMT <- All_Prob$CM*All_Prob$Therm
All_Prob$CPT <- All_Prob$CP*All_Prob$Therm
All_Prob$CAMP <- All_Prob$CAM*All_Prob$P
All_Prob$CAMT <- All_Prob$CAM*All_Prob$Therm
All_Prob$CAPT <- All_Prob$CAP*All_Prob$Therm
All_Prob$CMPT <- All_Prob$CMP*All_Prob$Therm
All_Prob$CAMPT <- All_Prob$CAMT*All_Prob$Therm

# f) Normalizing the lake-specific probability matrices and adding the null model

Normal_Matrix <- setDT(All_Prob)[, .(Predator = Predator, Prey = Prey, Interaction = Interaction, Norm_A = A/sum(A), Norm_M = M/sum(M), Norm_P = P/sum(P), Norm_T = Therm/sum(Therm), Norm_AM = AM/sum(AM), Norm_AP = AP/sum(AP), Norm_AT = AT/sum(AT), Norm_MP = MP/sum(MP), Norm_MT = MT/sum(MT), Norm_PT =PT/sum(PT),
                                     Norm_AMP = AMP/sum(AMP), Norm_AMT = AMT/sum(AMT), Norm_APT = APT/sum(APT), Norm_MPT = MPT/sum(MPT), Norm_AMPT = AMPT/sum(AMPT), Norm_C = C/sum(C), Norm_CA = CA/sum(CA), Norm_CM = CM/sum(CM), Norm_CP = CP/sum(CP), Norm_CT = CT/sum(CT), 
                                     Norm_CAM = CAM/sum(CAM), Norm_CAP = CAP/sum(CAP), Norm_CAT = CAT/sum(CAT), Norm_CMP = CMP/sum(CMP), Norm_CMT = CMT/sum(CMT), Norm_CPT = CPT/sum(CPT), Norm_CAMP = CAMP/sum(CAMP), Norm_CAMT = CAMT/sum(CAMT), Norm_CAPT = CAPT/sum(CAPT), 
                                     Norm_CMPT = CMPT/sum(CMPT), Norm_CAMPT = CAMPT/sum(CAMPT), New_R = Interaction/sum(Interaction)), by = Lake_ID]

All_Prob_Norm <- Normal_Matrix

# g) Adding the observed interactions

# Import the lake-specific interaction matrix where interactions are observed from stomach content analyses
Int_Matrix <- read.csv("Lake_Specific_Interaction_Matrix.csv", header = TRUE)

# This file contains the lake-specific interactions observed in the stomach content analysis during the 
# Broadscale Monitoring Program 2008-2012.
#
# Data details:
#  [1] Lake_ID: The unique Lake ID for each of the lakes used in the study
#  [2] Predator: common name for the consumer in the interaction
#  [3] Prey: common name for the prey in the interaction


# h) Removing rare interactions (see Appendix S2)
# -----------------------------------------------
#tempMatrix <- data.frame(character(), character(), character(), character())
#names(tempMatrix) <- names(Int_Matrix)

#for (i in 1:length(unique(Int_Matrix$Lake_ID))){
#  tempLake <- subset(Int_Matrix, Int_Matrix$Lake_ID == unique(Int_Matrix$Lake_ID)[i])
#  for (j in 1:length(unique(tempLake$Predator))){
#    tempPred <- subset(tempLake, tempLake$Predator == unique(tempLake$Predator)[j])
#    for (k in 1:length(unique(tempPred$Prey))){
#      tempPrey <- subset(tempPred, tempPred$Prey == unique(tempPred$Prey)[k])
#      if (nrow(tempPrey) > 1){
#        tempMatrix <- rbind(tempMatrix, tempPrey)
#      }
#    }
#  }

#}
#Int_Matrix <- tempMatrix
# -----------------------------------------------


Int_Matrix$Int_Obs <- 1
All_Prob_Norm <- merge(All_Prob_Norm, Int_Matrix, by = c("Lake_ID", "Predator", "Prey"), all.x = TRUE)
All_Prob_Norm[is.na(All_Prob_Norm)] <- 0
All_Prob_Norm <- unique(All_Prob_Norm)

#Assigns a unique an ID on all the possible interactions for each lake so that the resample algorithm can select interactions based
#on them 
tempDT <- setDT(All_Prob_Norm)[, .(ID = seq.int(Prey), Prey = Prey, Predator = Predator), by = c("Lake_ID")]
All_Prob_Norm <- merge(as.data.frame(tempDT), as.data.frame(All_Prob_Norm), by = c("Lake_ID", "Predator", "Prey"))
All_Prob_Norm <- setDT(All_Prob_Norm)

## Part 3 - Selecting interactions based on the ecological structuring mechanisms and comparing to the observed interactions
## -------------------------------------------------------------------------------------------------------------------------
All_Prob_Norm_names <- c("Lake_ID", "Predator", "Prey", "ID", "Interaction", "Norm_A", "Norm_M", "Norm_P", "Norm_T", "Norm_AM", "Norm_AP",
                         "Norm_AT", "Norm_MP", "Norm_MT", "Norm_PT", "Norm_AMP", "Norm_AMT", "Norm_APT", "Norm_MPT", "Norm_AMPT", "Norm_C",
                         "Norm_CA", "Norm_CM", "Norm_CP", "Norm_CT", "Norm_CAM", "Norm_CAP", "Norm_CAT", "Norm_CMP", "Norm_CMT", "Norm_CPT",
                         "Norm_CAMP", "Norm_CAMT", "Norm_CAPT", "Norm_CMPT", "Norm_CAMPT", "New_R", "Int_Obs")

A_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
A_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, A_Predicted, "Norm_A")

names(All_Prob_Norm) <- All_Prob_Norm_names
M_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
M_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, M_Predicted, "Norm_M")

names(All_Prob_Norm) <- All_Prob_Norm_names
P_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
P_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, P_Predicted, "Norm_P")

names(All_Prob_Norm) <- All_Prob_Norm_names
T_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
T_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, T_Predicted, "Norm_T")

names(All_Prob_Norm) <- All_Prob_Norm_names
AM_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
AM_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, AM_Predicted, "Norm_AM")

names(All_Prob_Norm) <- All_Prob_Norm_names
AP_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
AP_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, AP_Predicted, "Norm_AP")

names(All_Prob_Norm) <- All_Prob_Norm_names
AT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
AT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, AT_Predicted, "Norm_AT")

names(All_Prob_Norm) <- All_Prob_Norm_names
MP_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
MP_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, MP_Predicted, "Norm_MP")

names(All_Prob_Norm) <- All_Prob_Norm_names
MT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
MT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, MT_Predicted, "Norm_MT")

names(All_Prob_Norm) <- All_Prob_Norm_names
PT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
PT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, PT_Predicted, "Norm_PT")

names(All_Prob_Norm) <- All_Prob_Norm_names
AMP_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
AMP_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, AMP_Predicted, "Norm_AMP")

names(All_Prob_Norm) <- All_Prob_Norm_names
AMT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
AMT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, AMT_Predicted, "Norm_AMT")

names(All_Prob_Norm) <- All_Prob_Norm_names
APT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
APT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, APT_Predicted, "Norm_APT")

names(All_Prob_Norm) <- All_Prob_Norm_names
MPT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
MPT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, MPT_Predicted, "Norm_MPT")

names(All_Prob_Norm) <- All_Prob_Norm_names
AMPT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
AMPT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, AMPT_Predicted, "Norm_AMPT")

names(All_Prob_Norm) <- All_Prob_Norm_names
C_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
C_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, C_Predicted, "Norm_C")

names(All_Prob_Norm) <- All_Prob_Norm_names
CA_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CA_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CA_Predicted, "Norm_CA")

names(All_Prob_Norm) <- All_Prob_Norm_names
CM_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CM_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CM_Predicted, "Norm_CM")

names(All_Prob_Norm) <- All_Prob_Norm_names
CP_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CP_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CP_Predicted, "Norm_CP")

names(All_Prob_Norm) <- All_Prob_Norm_names
CT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CT_Predicted, "Norm_CT")

names(All_Prob_Norm) <- All_Prob_Norm_names
CAM_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CAM_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CAM_Predicted, "Norm_CAM")

names(All_Prob_Norm) <- All_Prob_Norm_names
CAP_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CAP_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CAP_Predicted, "Norm_CAP")

names(All_Prob_Norm) <- All_Prob_Norm_names
CAT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CAT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CAT_Predicted, "Norm_CAT")

names(All_Prob_Norm) <- All_Prob_Norm_names
CMP_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CMP_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CMP_Predicted, "Norm_CMP")

names(All_Prob_Norm) <- All_Prob_Norm_names
CMT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CMT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CMT_Predicted, "Norm_CMT")

names(All_Prob_Norm) <- All_Prob_Norm_names
CPT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CPT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CPT_Predicted, "Norm_CPT")

names(All_Prob_Norm) <- All_Prob_Norm_names
CAMP_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CAMP_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CAMP_Predicted, "Norm_CAMP")

names(All_Prob_Norm) <- All_Prob_Norm_names
CAMT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CAMT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CAMT_Predicted, "Norm_CAMT")

names(All_Prob_Norm) <- All_Prob_Norm_names
CAPT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CAPT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CAPT_Predicted, "Norm_CAPT")

names(All_Prob_Norm) <- All_Prob_Norm_names
CMPT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CMPT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CMPT_Predicted, "Norm_CMPT")

names(All_Prob_Norm) <- All_Prob_Norm_names
CAMPT_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double(), double(), double())
CAMPT_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, CAMPT_Predicted, "Norm_CAMPT")

names(All_Prob_Norm) <- All_Prob_Norm_names
R_Predicted <- data.frame(character(), double(), double(), double(), double(), double(), double())
R_Predicted <- TSS_Calc_Compiled(All_Prob_Norm, R_Predicted, "New_R")

A_Average <- setDT(A_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
M_Average <- setDT(M_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
P_Average <- setDT(P_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
T_Average <- setDT(T_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
AM_Average <- setDT(AM_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
AP_Average <- setDT(AP_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
AT_Average <- setDT(AT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
MP_Average <- setDT(MP_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
MT_Average <- setDT(MT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
PT_Average <- setDT(PT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
AMP_Average <- setDT(AMP_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
AMT_Average <- setDT(AMT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
APT_Average <- setDT(APT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
MPT_Average <- setDT(MPT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
AMPT_Average <- setDT(AMPT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]

C_Average <- setDT(C_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CP_Average <- setDT(CP_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CA_Average <- setDT(CA_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CM_Average <- setDT(CM_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CP_Average <- setDT(CP_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CT_Average <- setDT(CT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CAM_Average <- setDT(CAM_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CAP_Average <- setDT(CAP_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CAT_Average <- setDT(CAT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CMP_Average <- setDT(CMP_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CMT_Average <- setDT(CMT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CPT_Average <- setDT(CPT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CAMP_Average <- setDT(CAMP_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CAMT_Average <- setDT(CAMT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CAPT_Average <- setDT(CAPT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CMPT_Average <- setDT(CMPT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
CAMPT_Average <- setDT(CAMPT_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]
R_Average <- setDT(R_Predicted)[, .(u = mean(u), v = mean(v), w = mean(w), x = mean(x), TSS = mean(TSS), TSS_sd = sd(TSS), Sens = mean(Sens), Sens_sd = sd(Sens)), by = Lake_ID]

A_Average$Type <- "A"
M_Average$Type <- "M"
P_Average$Type <- "P"
T_Average$Type <- "T"
AM_Average$Type <- "AM"
AP_Average$Type <- "AP"
AT_Average$Type <- "AT"
MP_Average$Type <- "MP"
MT_Average$Type <- "MT"
PT_Average$Type <- "PT"
AMP_Average$Type <- "AMP"
AMT_Average$Type <- "AMT"
APT_Average$Type <- "APT"
MPT_Average$Type <- "MPT"
AMPT_Average$Type <- "AMPT"

C_Average$Type <- "C"
CA_Average$Type <- "CA"
CM_Average$Type <- "CM"
CP_Average$Type <- "CP"
CT_Average$Type <- "CT"
CAM_Average$Type <- "CAM"
CAP_Average$Type <- "CAP"
CAT_Average$Type <- "CAT"
CMP_Average$Type <- "CMP"
CMT_Average$Type <- "CMT"
CPT_Average$Type <- "CPT"
CAMP_Average$Type <- "CAMP"
CAMT_Average$Type <- "CAMT"
CAPT_Average$Type <- "CAPT"
CMPT_Average$Type <- "CMPT"
CAMPT_Average$Type <- "CAMPT"
R_Average$Type <- "R"

All_Average <- rbind(A_Average, M_Average, P_Average, T_Average,  C_Average, AM_Average, AP_Average, AT_Average, MP_Average, MT_Average, PT_Average,
                     AMP_Average, AMT_Average, APT_Average, MPT_Average, AMPT_Average, R_Average, CP_Average, CA_Average, CM_Average, 
                     CP_Average, CT_Average, CAM_Average, CAP_Average, CAT_Average, CAPT_Average, CMP_Average, CMT_Average, CPT_Average, CAMP_Average, 
                     CAMT_Average, CPT_Average, CMPT_Average, CAMPT_Average)

#Create the table reporting mean and standard deviations of the mean TSS and Sens scores
All_Table <- setDT(All_Average)[, .(TSS = format(round(mean(TSS),3), nsmall= 2), Sens = format(round(mean(Sens),3), nsmall = 2), Sens_sd = format(round(mean(Sens_sd),3), nsmall = 2), TSS_sd = format(round(mean(TSS_sd),3),nsmall = 2)), by = Type]

## Part 4 - Calculating interaction turnover between lakes and creating Beta boxplots
## ----------------------------------------------------------------------------------

#a) Calculating interaction turnver

All_Int <- data.frame(All_Prob_Norm)[,c(1:3,38)]
Combinations <- length(unique(All_Int$Lake_ID)) * (length(unique(All_Int$Lake_ID)) - 1)/2 
Calcs <- data.frame(matrix(ncol = 11, nrow = Combinations)) 
names(Calcs) <- c("Lake_A", "Lake_B", "a", "b", "c", "Rewired", "Novel", "Basal", "Top", "Intermediate", "Num_Diff_Species")
Calcs <- Beta_Calc_Compile(Calcs, All_Int)

Calcs$a <- as.double(Calcs$a)
Calcs$b <- as.double(Calcs$b)
Calcs$c <- as.double(Calcs$c)
Calcs$Rewired <- as.double(Calcs$Rewired)
Calcs$Novel <- as.double(Calcs$Novel)
Calcs$Top <- as.double(Calcs$Top)
Calcs$Basal <- as.double(Calcs$Basal)
Calcs$Intermediate <- as.double(Calcs$Intermediate)
Calcs$Num_Diff_Species <- as.double(Calcs$Num_Diff_Species)

Int_Turnover <- B_Calcs_compiled(Calcs)

# b) Creating the beta boxplots

# Code to create Figure 4.
Bint <- Int_Turnover[,c(1:2,13)]
names(Bint) <- c("X", "Iter", "Value")
Bint$Type <- "INT"
Brw <- Int_Turnover[,c(1:2,12)]
Brw$Type <- "RW"
names(Brw) <- names(Bint)
Bst <- Int_Turnover[,c(1:2,14)]
Bst$Type <- "ST"
names(Bst) <- names(Bint)
Bprey <- Int_Turnover[,c(1:2,15)]
Bprey$Type <- "Basal"
names(Bprey) <- names(Bint)
Bpred <- Int_Turnover[,c(1:2,16)]
Bpred$Type <- "Top"
names(Bpred) <- names(Bint)
Bintermediate <- Int_Turnover[,c(1:2,17)]
Bintermediate$Type <- "Intermediate"
names(Bintermediate) <- names(Bint)
Bnovel <- Int_Turnover[,c(1:2,18)]
Bnovel$Type <- "Novel"
names(Bnovel) <- names(Bint)

B_Int <- rbind(Bint, Brw, Bst, Bprey, Bintermediate, Bpred,Bnovel)
B_Int$Type2 <- factor(B_Int$Type,
                       levels = c('INT', 'RW','ST', "Basal", "Intermediate", "Top", "Novel"),ordered = TRUE)

p1 <- ggplot(aes(y = Value, x = factor(Type2)), data = B_Int) 
p1 <- p1 + stat_summary(fun.data = min.mean.sd.max, geom = "boxplot", fill = c("white", "grey90", "grey90", "grey70", "grey70", "grey70", "grey70"), position=position_dodge2(width=1.5)) +
  ylab("Beta") + xlab("") + 
  ylim(0,1) +
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))


## Part 5 - Evaluating environmental drivers of interaction turnover
## -----------------------------------------------------------------

## Calculating species richness at each lake
spec_rich <- data.frame(matrix(nrow = length(unique(Lake_Environment$Lake_ID)), ncol = 2))
for (j in 1:length(unique(All_Int$Lake_ID))){
  lake <- unique(All_Int$Lake_ID)[j]
  lake_sub <- subset(All_Int, All_Int$Lake_ID == lake)
  n_spec <- length(unique(union(lake_sub$Predator, lake_sub$Prey)))
  spec_rich[j,] <- c(as.character(lake), n_spec)
}
names(spec_rich) <- c("Lake_ID", "n_Spec")
Lake_Environment <- merge(Lake_Environment, spec_rich, by = "Lake_ID")

## Calculating morphoedaphic index
Lake_Environment$MEI <- Lake_Environment$TDS_mgL/Lake_Environment$Depmn_m

Lake_Distance <- data.frame(matrix(ncol = 2, nrow = length(unique(Lake_Environment$Lake_ID)) * (length(unique(Lake_Environment$Lake_ID)) - 1)/2))
index <- 1
names(Lake_Distance) <- c("Lake_A", "Lake_B")
for (i in 1:(nrow(Lake_Environment)-1)){
  Lake_A <- Lake_Environment$Lake_ID[i]
  for (j in (i+1):nrow(Lake_Environment)){
    Lake_B <- Lake_Environment$Lake_ID[j]
    Lake_Distance[index,] <- c(as.character(Lake_A), as.character(Lake_B))
    index <- index + 1
  }
}

Lake_Int_full <- merge(Lake_Distance, Int_Turnover, by = c("Lake_A", "Lake_B"))
Lake_Int_full2 <- merge(Lake_Distance, Int_Turnover, by= c("Lake_A", "Lake_B"))
Lake_Int_full <- rbind(Lake_Int_full, Lake_Int_full2)

#Setting up the beta scores as dissimilarity objects.

B_int_dist <- data.frame(matrix(nrow = length(unique(Lake_Environment$Lake_ID)), ncol = length(unique(Lake_Environment$Lake_ID))))
row.names(B_int_dist) <- unique(Lake_Environment$Lake_ID)
names(B_int_dist) <- unique(Lake_Environment$Lake_ID)

for (i in 1:nrow(Lake_Int_full)){
  Lake_A <- Lake_Int_full$Lake_A[i]
  Lake_B <- Lake_Int_full$Lake_B[i]
  B_int_dist[Lake_B, Lake_A] <- Lake_Int_full$Bint[i]
}
B_int_dist[is.na(B_int_dist)]<- 0
B_dist <- as.dist(B_int_dist)

#Rewiring

B_rw_dist <- data.frame(matrix(nrow = length(unique(Lake_Environment$Lake_ID)), ncol = length(unique(Lake_Environment$Lake_ID))))
row.names(B_rw_dist) <- unique(Lake_Environment$Lake_ID)
names(B_rw_dist) <- unique(Lake_Environment$Lake_ID)

for (i in 1:nrow(Lake_Int_full)){
  Lake_A <- Lake_Int_full$Lake_A[i]
  Lake_B <- Lake_Int_full$Lake_B[i]
  B_rw_dist[Lake_B, Lake_A] <- Lake_Int_full$Brw[i]
}
B_rw_dist[is.na(B_rw_dist)]<- 0
B_rw_dist <- as.dist(B_rw_dist)

#Species Turnover

B_st_dist <- data.frame(matrix(nrow = length(unique(Lake_Environment$Lake_ID)), ncol = length(unique(Lake_Environment$Lake_ID))))
row.names(B_st_dist) <- unique(Lake_Environment$Lake_ID)
names(B_st_dist) <- unique(Lake_Environment$Lake_ID)

for (i in 1:nrow(Lake_Int_full)){
  Lake_A <- Lake_Int_full$Lake_A[i]
  Lake_B <- Lake_Int_full$Lake_B[i]
  B_st_dist[Lake_B, Lake_A] <- Lake_Int_full$Bst[i]
}
B_st_dist[is.na(B_st_dist)]<- 0
B_st_dist <- as.dist(B_st_dist)

#basal

B_b_dist <- data.frame(matrix(nrow = length(unique(Lake_Environment$Lake_ID)), ncol = length(unique(Lake_Environment$Lake_ID))))
row.names(B_b_dist) <- unique(Lake_Environment$Lake_ID)
names(B_b_dist) <- unique(Lake_Environment$Lake_ID)

for (i in 1:nrow(Lake_Int_full)){
  Lake_A <- Lake_Int_full$Lake_A[i]
  Lake_B <- Lake_Int_full$Lake_B[i]
  B_b_dist[Lake_B, Lake_A] <- Lake_Int_full$Bb[i]
}
B_b_dist[is.na(B_b_dist)]<- 0
B_b_dist <- as.dist(B_b_dist)

#intermediate

B_i_dist <- data.frame(matrix(nrow = length(unique(Lake_Environment$Lake_ID)), ncol = length(unique(Lake_Environment$Lake_ID))))
row.names(B_i_dist) <- unique(Lake_Environment$Lake_ID)
names(B_i_dist) <- unique(Lake_Environment$Lake_ID)

for (i in 1:nrow(Lake_Int_full)){
  Lake_A <- Lake_Int_full$Lake_A[i]
  Lake_B <- Lake_Int_full$Lake_B[i]
  B_i_dist[Lake_B, Lake_A] <- Lake_Int_full$Bintermediate[i]
}
B_i_dist[is.na(B_i_dist)]<- 0
B_i_dist <- as.dist(B_i_dist)

#top

B_t_dist <- data.frame(matrix(nrow = length(unique(Lake_Environment$Lake_ID)), ncol = length(unique(Lake_Environment$Lake_ID))))
row.names(B_t_dist) <- unique(Lake_Environment$Lake_ID)
names(B_t_dist) <- unique(Lake_Environment$Lake_ID)

for (i in 1:nrow(Lake_Int_full)){
  Lake_A <- Lake_Int_full$Lake_A[i]
  Lake_B <- Lake_Int_full$Lake_B[i]
  B_t_dist[Lake_B, Lake_A] <- Lake_Int_full$Bt[i]
}
B_t_dist[is.na(B_t_dist)]<- 0
B_t_dist <- as.dist(B_t_dist)

#novel

B_n_dist <- data.frame(matrix(nrow = length(unique(Lake_Environment$Lake_ID)), ncol = length(unique(Lake_Environment$Lake_ID))))
row.names(B_n_dist) <- unique(Lake_Environment$Lake_ID)
names(B_n_dist) <- unique(Lake_Environment$Lake_ID)

for (i in 1:nrow(Lake_Int_full)){
  Lake_A <- Lake_Int_full$Lake_A[i]
  Lake_B <- Lake_Int_full$Lake_B[i]
  B_n_dist[Lake_B, Lake_A] <- Lake_Int_full$Bnovel[i]
}
B_n_dist[is.na(B_n_dist)]<- 0
B_n_dist <- as.dist(B_n_dist)

#Adonis Analysis
B_int <- adonis(B_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, permutations = 4999)
B_rw <- adonis(B_rw_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, permutations = 4999)
B_st <- adonis(B_st_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, permutations = 4999)

B_basal <- adonis(B_b_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, permutations = 4999)
B_intermediate <- adonis(B_i_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, permutations = 4999)
B_top <- adonis(B_t_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, permutations = 4999)
B_novel <- adonis(B_n_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, permutations = 4999)


#Adonis Analysis with North/South drainage included
B_int <- adonis(B_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, strata = Lake_Environment$Drainage, permutations = 4999)
B_rw <- adonis(B_rw_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, strata = Lake_Environment$Drainage, permutations = 4999)
B_st <- adonis(B_st_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, strata = Lake_Environment$Drainage, permutations = 4999)

B_basal <- adonis(B_b_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, strata = Lake_Environment$Drainage, permutations = 4999)
B_intermediate <- adonis(B_i_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, strata = Lake_Environment$Drainage, permutations = 4999)
B_top <- adonis(B_t_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, strata = Lake_Environment$Drainage, permutations = 4999)
B_novel <- adonis(B_n_dist~Latitude_DD+Area_ha+Depmn_m + pH + MEI + TP + DD5_8110 + n_Spec, Lake_Environment, strata = Lake_Environment$Drainage, permutations = 4999)


## Part 6 - Determining species dissimilarity and creating dissimilarity violin plots.
## -----------------------------------------------------------------------------------

All_Prob_Norm <- data.frame(All_Prob_Norm)

#Only care about the observed interactions, so subset the matrix to only the lake ID, Predator Species, Prey Species, and 
#column indicating whether or not an interaction was observed
All_Prob_Sub <- All_Prob_Norm[,c(1,2,3,38)]

#Only care about those interactions which have actually occurred, not all possible interactions in that lake, so subset
#for only interactions which have been observed
All_Prob_Sub <- subset(All_Prob_Sub, All_Prob_Sub$Int_Obs > 0) 

#There are 30 different possible species positions.
Species_Position <- data.frame(matrix(ncol = 32))
names(Species_Position) <- c("Lake_ID", "Species", c(1:30))
index <- 1

#Counts the number of times each species in each lake was observed at each unique species position
for (lake_No in 1:length(unique(All_Prob_Sub$Lake_ID))){
  Lake <- as.character(unique(All_Prob_Sub$Lake_ID)[lake_No])
  Lake_Sub <- subset(All_Prob_Sub, All_Prob_Sub$Lake_ID == Lake)
  
  Pred_List <- as.character(unique(Lake_Sub$Predator))
  Prey_List <- as.character(unique(Lake_Sub$Prey))
  
  for (i in 1:length(Pred_List)){
    Pred_sub <- subset(Lake_Sub, as.character(Lake_Sub$Predator) == as.character(Pred_List)[i])
    for (j in 1:nrow(Pred_sub)){
      if (Pred_sub$Prey[j] %in% Pred_List){
        Prey_sub <- subset(Lake_Sub, as.character(Lake_Sub$Predator) == as.character(Pred_sub$Prey[j]))
        for (k in 1:nrow(Prey_sub)){
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(Pred_sub$Predator[j])
          Species_Position[index, 5] <- 1
          index <- index + 1
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(Prey_sub$Predator[k])
          Species_Position[index, 4] <- 1
          index <- index + 1
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(Prey_sub$Prey[k])
          Species_Position[index, 3] <- 1
          index <- index + 1
          Prey1 <- as.character(subset(Lake_Sub, as.character(Lake_Sub$Predator) == as.character(Prey_sub$Predator)[j])$Prey)
          Prey2 <- as.character(subset(Lake_Sub, as.character(Lake_Sub$Predator) == as.character(Prey_sub$Prey[k]))$Prey)
          if (as.character(Prey_sub$Predator)[j] %in% Prey2 & as.character(Prey_sub$Prey[k]) %in% Prey1){
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Predator[j])
            Species_Position[index, 19] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Prey_sub$Predator[k])
            Species_Position[index, 20] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Prey_sub$Prey[k])
            Species_Position[index, 18] <- 1
            index <- index + 1
          }
          if (as.character(Pred_sub$Predator[j]) %in% Prey1) {
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Predator[j])
            Species_Position[index, 23] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Prey_sub$Predator[k])
            Species_Position[index, 21] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Prey_sub$Prey[k])
            Species_Position[index, 22] <- 1
            index <- index + 1
          }
          if (as.character(Prey_sub$Prey[k] %in% as.character(Pred_sub$Prey))){
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Predator[j])
            Species_Position[index, 7] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Prey_sub$Predator[k])
            Species_Position[index, 8] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Prey_sub$Prey[k])
            Species_Position[index, 6] <- 1
            index <- index + 1
          }
          if (as.character(Prey_sub$Prey[k]) %in% Pred_List){
            Prey_sub_sub <- subset(Lake_Sub, as.character(Lake_Sub$Predator) == as.character(Prey_sub$Prey[k]))
            if (as.character(Pred_sub$Predator[j]) %in% as.character(Prey_sub_sub$Prey)){
              Species_Position[index,1] <- Lake
              Species_Position[index,2] <- as.character(Pred_sub$Predator[j])
              Species_Position[index, 9] <- 1
              index <- index + 1
              Species_Position[index,1] <- Lake
              Species_Position[index,2] <- as.character(Prey_sub$Predator[k])
              Species_Position[index, 9] <- 1
              index <- index + 1
              Species_Position[index,1] <- Lake
              Species_Position[index,2] <- as.character(Prey_sub$Prey[k])
              Species_Position[index, 9] <- 1
              index <- index + 1
              if (as.character(Prey_sub$Prey[k]) %in% as.character(Pred_sub$Prey)){
                Species_Position[index,1] <- Lake
                Species_Position[index,2] <- as.character(Pred_sub$Predator[j])
                Species_Position[index, 26] <- 1
                index <- index + 1
                Species_Position[index,1] <- Lake
                Species_Position[index,2] <- as.character(Prey_sub$Predator[k])
                Species_Position[index, 25] <- 1
                index <- index + 1
                Species_Position[index,1] <- Lake
                Species_Position[index,2] <- as.character(Prey_sub$Prey[k])
                Species_Position[index, 24] <- 1
                index <- index + 1
                if (as.character(Pred_sub$Predator[j]) %in% as.character(Prey_sub$Prey)){
                  Species_Position[index,1] <- Lake
                  Species_Position[index,2] <- as.character(Pred_sub$Predator[j])
                  Species_Position[index, 29] <- 1
                  index <- index + 1
                  Species_Position[index,1] <- Lake
                  Species_Position[index,2] <- as.character(Prey_sub$Predator[k])
                  Species_Position[index, 30] <- 1
                  index <- index + 1
                  Species_Position[index,1] <- Lake
                  Species_Position[index,2] <- as.character(Prey_sub$Prey[k])
                  Species_Position[index, 28] <- 1
                  index <- index + 1
                  if (as.character(Prey_sub$Predator[k]) %in% as.character(Prey_sub_sub$Prey)){
                    Species_Position[index,1] <- Lake
                    Species_Position[index,2] <- as.character(Pred_sub$Predator[j])
                    Species_Position[index, 27] <- 1
                    index <- index + 1
                    Species_Position[index,1] <- Lake
                    Species_Position[index,2] <- as.character(Prey_sub$Predator[k])
                    Species_Position[index, 27] <- 1
                    index <- index + 1
                    Species_Position[index,1] <- Lake
                    Species_Position[index,2] <- as.character(Prey_sub$Prey[k])
                    Species_Position[index, 27] <- 1
                    index <- index + 1
                  }
                }
              }
            }
          }
        }
      }
    }
    if (nrow(Pred_sub)>1){
      for (l in 1:(length(Pred_sub$Prey)-1)){
        for (m in (l+1):length(Pred_sub$Prey)) {
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(Pred_sub$Predator[1])
          Species_Position[index, 11] <- 1
          index <- index + 1
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(Pred_sub$Prey[l])
          Species_Position[index, 10] <- 1
          index <- index + 1
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(Pred_sub$Prey[m])
          Species_Position[index, 10] <- 1
          index <- index + 1
          
          Prey1 <- as.character(unique(subset(Lake_Sub, Lake_Sub$Predator == as.character(Pred_sub$Prey[l]))$Prey))
          Prey2 <- as.character(unique(subset(Lake_Sub, Lake_Sub$Predator == as.character(Pred_sub$Prey[m]))$Prey))
          if (as.character(Pred_sub$Predator[1]) %in% Prey1 & as.character(Pred_sub$Predator[1]) %in% Prey2){
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Predator[1])
            Species_Position[index, 32] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Prey[l])
            Species_Position[index, 31] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Prey[m])
            Species_Position[index, 31] <- 1
            index <- index + 1
          }
          if (as.character(Pred_sub$Prey[l]) %in% Prey2 & as.character(Pred_sub$Prey[m]) %in% Prey1){
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Predator[1])
            Species_Position[index, 15] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Prey[l])
            Species_Position[index, 14] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(Pred_sub$Prey[m])
            Species_Position[index, 14] <- 1
            index <- index + 1
          }
        }
      }
    }
  }
  for (n in 1:length(Prey_List)){
    PreyM <- subset(Lake_Sub, Lake_Sub$Prey == Prey_List[n])
    if (nrow(PreyM)>1){
      for (o in 1:(nrow(PreyM)-1)){
        for (p in (o+1):nrow(PreyM)){
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(PreyM$Predator[o])
          Species_Position[index, 12] <- 1
          index <- index + 1
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(PreyM$Predator[p])
          Species_Position[index, 12] <- 1
          index <- index + 1
          Species_Position[index,1] <- Lake
          Species_Position[index,2] <- as.character(PreyM$Prey[o])
          Species_Position[index, 13] <- 1
          index <- index + 1
          Pred1 <- unique(subset(Lake_Sub, Lake_Sub$Pred == as.character(PreyM$Predator[o]))$Prey)
          Pred2 <- unique(subset(Lake_Sub, Lake_Sub$Pred == as.character(PreyM$Predator[p]))$Prey)
          if (as.character(PreyM$Predator[o]) %in% Prey2 & as.character(PreyM$Predator[p]) %in% Prey1){
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(PreyM$Predator[o])
            Species_Position[index, 16] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(PreyM$Predator[p])
            Species_Position[index, 16] <- 1
            index <- index + 1
            Species_Position[index,1] <- Lake
            Species_Position[index,2] <- as.character(PreyM$Prey[o])
            Species_Position[index, 17] <- 1
            index <- index + 1
          }
        }
      }
    }
  }
}


# Need to normalize each count of species roles by the sum of all frequencies to control for the total number of 
# species-specific interactions
Species_Position[is.na(Species_Position)] <- 0
Species_Pos <- setDT(Species_Position)[,.(Role_1 = sum(`1`), Role_2 = sum(`2`), Role_3= sum(`3`), Role_4 = sum(`4`), Role_5 = sum(`5`),
                                          Role_6 = sum(`6`), Role_7 = sum(`7`), Role_8 = sum(`8`), Role_9 = sum(`9`), Role_10 = sum(`10`),
                                          Role_11 = sum(`11`), Role_12 = sum(`12`), Role_13 = sum(`13`), Role_14 = sum(`14`), Role_15 = sum(`15`),
                                          Role_16 = sum(`16`), Role_17 = sum(`17`), Role_18 = sum(`18`), Role_19 = sum(`19`), Role_20 = sum(`20`),
                                          Role_21 = sum(`21`), Role_22 = sum(`22`), Role_23 = sum(`23`), Role_24 = sum(`24`), Role_25 = sum(`25`), 
                                          Role_26 = sum(`26`), Role_27 = sum(`27`), Role_28 = sum(`28`), Role_29 = sum(`29`), Role_30 = sum(`30`)), by = c("Lake_ID", "Species")]
Spec <- as.data.frame(Species_Pos)
Species_Pos$Sum <- rowSums(Spec[,c(3:32)])
Species_Pos <- setDT(Species_Pos)[,.(Role_1 = Role_1/Sum, Role_2 = Role_1/Sum, Role_3= Role_3/Sum, Role_4 = Role_4/Sum, Role_5 = Role_5/Sum,
                                     Role_6 = Role_6/Sum, Role_7 = Role_7/Sum, Role_8 = Role_8/Sum, Role_9 = Role_9/Sum, Role_10 = Role_10/Sum,
                                     Role_11 = Role_11/Sum, Role_12 = Role_12/Sum, Role_13 = Role_13/Sum, Role_14 = Role_14/Sum, Role_15 = Role_15/Sum,
                                     Role_16 = Role_16/Sum, Role_17 = Role_17/Sum, Role_18 = Role_18/Sum, Role_19 = Role_19/Sum, Role_20 = Role_20/Sum,
                                     Role_21 = Role_21/Sum, Role_22 = Role_22/Sum, Role_23 = Role_23/Sum, Role_24 = Role_24/Sum, Role_25 = Role_25/Sum, 
                                     Role_26 = Role_26/Sum, Role_27 = Role_27/Sum, Role_28 = Role_28/Sum, Role_29 = Role_29/Sum, Role_30 = Role_30/Sum), by = c("Lake_ID", "Species")]


# Modified Permanova as presented in Baker et al. Ecography
# Citation: Baker, N.J., Kaartinen, R., Roslin, T. & Stouffer, D.B. (2015). 
#           Species' roles in food webs show fidelity across a highly variable 
#           oak forest. Ecography, 38, 130-139

Species_Pos <- data.frame(Species_Pos)

# Need to calculate the observed dissimilarity for each species to compare to the randomly reshuffled ones.
Species_List <- sort(as.character(unique(Species_Pos$Species)))
Dw <- data.frame(matrix(nrow = 1, ncol = 31))
names(Dw) <- c(as.character(Species_List))
Dw_actual <- Dw_Calc_Compiled(Species_Pos, Dw)
Dw_actual[is.na(Dw_actual)] <- 0
Dw_actual$Total <- rowSums(Dw_actual)
Doverall <- Dist_Calc_Compiled(Species_Pos)
Fst <- (Doverall-Dw_actual$Total[1])/(length(Species_List) - 1)/(Dw_actual$Total[1]/(nrow(Species_Pos)-length(Species_List)))

# Need to reshuffle species identities within lakes for null model. Here the reshuffling occurs 5000 times and is saved to a new matrix
# creating a large matrix which will then be subset to perform the distance calculations on.
Species_Pos2 <- data.frame(matrix(nrow = 1, ncol = ncol(Species_Pos)+1))
names(Species_Pos2)<- c(names(Species_Pos), "Iter")
for (index in 1:5000){
  print(index)
  new_Species_Pos <- data.frame(matrix(ncol=ncol(Species_Pos)))
  names(new_Species_Pos) <- names(Species_Pos)
  for (lake in 1:length(unique(Species_Pos$Lake_ID))){
    Lake_Sub <- subset(Species_Pos, Species_Pos$Lake_ID == unique(Species_Pos$Lake_ID)[lake])
    rand_num <- sample(nrow(Lake_Sub), nrow(Lake_Sub), replace = FALSE)
    Species_List <- unique(Lake_Sub$Species)
    for (row in 1:nrow(Lake_Sub)){
      Lake_Sub$Species[row] <- Species_List[rand_num[row]]
    }
    new_Species_Pos <- rbind(new_Species_Pos, Lake_Sub)
  }
  new_Species_Pos <- new_Species_Pos[2:nrow(new_Species_Pos),]
  new_Species_Pos$Iter <- index
  Species_Pos2 <- rbind(Species_Pos2, new_Species_Pos)
}

## Alternatively, we can run a null model where we keep species degree constant:
## First need to calculate species degree:
species_degree <- data.frame(matrix(nrow = nrow(Species_Pos), ncol = 3))
names(species_degree) <- c("Lake_ID", "Species", "Degree")

i <- 1
for (lake in 1:length(unique(Species_Pos$Lake_ID))){
  Lake_Sub <- subset(All_Prob_Sub, All_Prob_Sub$Lake_ID ==  as.character(unique(Species_Pos$Lake_ID)[lake]))
  Lake_Sub_Spec_Pos <- subset(Species_Pos, as.character(Species_Pos$Lake_ID) == as.character(unique(Species_Pos$Lake_ID)[lake]))
  species_list <- unique(Lake_Sub_Spec_Pos$Species)
  pred_list <- unique(Lake_Sub$Predator)
  for (species in 1:length(species_list)){
    species_name <- as.character(species_list)[species]
    if (species_name %in% pred_list){
      pred_sub <- subset(Lake_Sub, Lake_Sub$Predator == species_name)
      species_degree$Degree[i] <- nrow(pred_sub)
      species_degree$Lake_ID[i] <-  as.character(unique(Species_Pos$Lake_ID)[lake])
      species_degree$Species[i] <- species_name
      i <- i + 1
    }
    else {
      species_degree$Degree[i] < - 0
      species_degree$Lake_ID[i] <-  as.character(unique(Species_Pos$Lake_ID)[lake])
      species_degree$Species[i] <- species_name
      i <- i+ 1
    }
  }
}
species_degree[is.na(species_degree)] <- 0
Species_Pos <- merge(species_degree, Species_Pos, by = c("Lake_ID", "Species"))

## and then run the null model generating all the matrices.
Species_Pos2 <- data.frame(matrix(nrow = 1, ncol = ncol(Species_Pos)+1))
names(Species_Pos2)<- c(names(Species_Pos), "Iter")
for (index in 1:5000){
  print(index)
  new_Species_Pos <- data.frame(matrix(ncol=ncol(Species_Pos)))
  names(new_Species_Pos) <- names(Species_Pos)
  for (lake in 1:length(unique(Species_Pos$Lake_ID))){
    Lake_Sub <- subset(Species_Pos, Species_Pos$Lake_ID == unique(Species_Pos$Lake_ID)[lake])
    Lake_Sub <- Lake_Sub[order(Lake_Sub$Degree),]
    Degree_list <- unique(Lake_Sub$Degree)
    i <- 1
    for (degree in 1:length(Degree_list)){
      degree_sub <- subset(Lake_Sub, Lake_Sub$Degree == unique(Lake_Sub$Degree)[degree])
      end <- nrow(degree_sub)
      rand_num <- sample(end, end, replace = FALSE)
      Species_List <- unique(degree_sub$Species)
      for (row in 1:nrow(degree_sub)){
        degree_sub$Species[row] <- Species_List[rand_num[row]]
      }
      new_Species_Pos <- rbind(new_Species_Pos, degree_sub)
      i <- i + end
    }
  }
  new_Species_Pos <- new_Species_Pos[2:nrow(new_Species_Pos),]
  new_Species_Pos$Iter <- index
  Species_Pos2 <- rbind(Species_Pos2, new_Species_Pos)
}

Species_Pos2 <- read.csv("C:/Users/Anne/Dropbox/PhD/Species_Role_Degree.csv", header = TRUE)

# Here the previously generated matrices are used and the dissimilarity of species in each iteration
# is calculated.
F_Matrix <- data.frame(matrix(nrow = 5000, ncol = 32))
Species_List <- sort(as.character(unique(Species_Pos$Species)))
names(F_Matrix) <- c(Species_List, "Iter")
for (iter in 1:5000){
  print(iter)
  new_Spec <- subset(Species_Pos2, Species_Pos2$Iter == iter)
  Species_List <- sort(as.character(unique(new_Spec$Species)))
  Dw <- data.frame(matrix(nrow = 1, ncol = 31))
  names(Dw) <- c(as.character(Species_List))
  Diter <- Dw_Calc_Compiled(new_Spec[,c(1:32)], Dw)
  Diter$Iter <- iter
  F_Matrix[iter,] <- Diter
}

write.csv(F_Matrix, "C:/Users/Anne/Dropbox/PhD/F_Matrix_Spec_Roles.csv")

F_Matrix$Type <- ""


BS <- F_Matrix[,c(3,33)]
BS$Type <- "Brook Stickleback"
names(BS) <- c("Value", "Type")
Burbot <- F_Matrix[,c(6,33)]
Burbot$Type <- "Burbot"
names(Burbot) <- c("Value", "Type")
Cisco <- F_Matrix[,c(7,33)]
Cisco$Type <- "Cisco"
names(Cisco) <- c("Value", "Type")
CS <- F_Matrix[,c(8,33)]
CS$Type <- "Common Shiner"
names(CS) <- c("Value", "Type")
JD <- F_Matrix[,c(10,33)]
JD$Type <- "Johnny Darter"
names(JD) <- c("Value", "Type")
LT <- F_Matrix[,c(12,33)]
LT$Type <- "Lake Trout"
names(LT) <- c("Value", "Type")
LW <- F_Matrix[,c(13,33)]
LW$Type <- "Lake Whitefish"
names(LW) <- c("Value", "Type")
LogP <- F_Matrix[,c(15,33)]
LogP$Type <- "Logperch"
names(LogP) <- c("Value", "Type")
NS <- F_Matrix[,c(18,33)]
NS$Type <- "Ninespine Stickleback"
names(NS) <- c("Value", "Type")
NP <- F_Matrix[,c(19,33)]
NP$Type <- "Northern Pike"
names(NP) <- c("Value", "Type")
RS <- F_Matrix[,c(22,33)]
RS$Type <- "Rainbow Smelt"
names(RS) <- c("Value", "Type")
RB <- F_Matrix[,c(23,33)]
RB$Type <- "Rock Bass"
names(RB) <- c("Value", "Type")
Sauger <- F_Matrix[,c(24,33)]
Sauger$Type <- "Sauger"
names(Sauger) <- c("Value", "Type")
SmBass <- F_Matrix[,c(26,33)]
SmBass$Type <- "Smallmouth Bass"
names(SmBass) <- c("Value", "Type")
SS <- F_Matrix[,c(27,33)]
SS$Type <- "Spottail Shiner"
names(SS) <- c("Value", "Type")
TP <- F_Matrix[,c(28,33)]
TP$Type <- "Trout-Perch"
names(TP) <- c("Value", "Type")
Wall <- F_Matrix[,c(29,33)]
Wall$Type <- "Walleye"
names(Wall) <- c("Value", "Type")
WS <- F_Matrix[,c(30,33)]
WS$Type <- "White Sucker"
names(WS) <- c("Value", "Type")
YP <- F_Matrix[,c(31,33)]
YP$Type <- "Yellow Perch"
names(YP) <- c("Value", "Type")

motif_Pred <- rbind(Wall, NP, LT, Burbot, YP, LW, Sauger, SmBass, Cisco)
motif_Pred$Type2 <- factor(motif_Pred$Type, as.character(motif_Pred$Type))
motif_Pred$Value <- as.double(motif_Pred$Value)

ggplot(motif_Pred, aes(Type2,Value)) +
  geom_violin(fill = "grey80") +
  ylab("Species Dissimilarity") + xlab("") + 
  geom_segment(aes(x = 0.8, y = mean(Dw_actual[1,29]), xend = 1.2, yend = mean(Dw_actual[1,29])))+
  geom_segment(aes(x = 1.8, y = mean(Dw_actual[1,19]), xend = 2.2, yend = mean(Dw_actual[1,19])))+
  geom_segment(aes(x = 2.8, y = mean(Dw_actual[1,12]), xend = 3.2, yend = mean(Dw_actual[1,12])))+
  geom_segment(aes(x = 3.8, y = mean(Dw_actual[1,6]), xend = 4.2, yend = mean(Dw_actual[1,6])))+
  geom_segment(aes(x = 4.8, y = mean(Dw_actual[1,31]), xend = 5.2, yend = mean(Dw_actual[1,31])))+
  geom_segment(aes(x = 5.8, y = mean(Dw_actual[1,13]), xend = 6.2, yend = mean(Dw_actual[1,13])))+
  geom_segment(aes(x = 6.8, y = mean(Dw_actual[1,24]), xend = 7.2, yend = mean(Dw_actual[1,24])))+
  geom_segment(aes(x = 7.8, y = mean(Dw_actual[1,26]), xend = 8.2, yend = mean(Dw_actual[1,26])))+
  geom_segment(aes(x = 8.8, y = mean(Dw_actual[1,7]), xend = 9.2, yend = mean(Dw_actual[1,7])))+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))

motif_Prey <- rbind(BS, CS, JD, LogP, NS, RS, RB, SS, TP, WS)
motif_Prey$Type2 <- factor(motif_Prey$Type, as.character(motif_Prey$Type))
motif_Prey$Value <- as.double(motif_Prey$Value)

ggplot(motif_Prey, aes(Type2,Value)) +
  geom_violin(fill = "grey80") +
  ylab("Species Dissimilarity") + xlab("") + 
  geom_segment(aes(x = 0.8, y = mean(Dw_actual[1,3]), xend = 1.2, yend = mean(Dw_actual[1,3])))+
  geom_segment(aes(x = 1.8, y = mean(Dw_actual[1,8]), xend = 2.2, yend = mean(Dw_actual[1,8])))+
  geom_segment(aes(x = 2.8, y = mean(Dw_actual[1,10]), xend = 3.2, yend = mean(Dw_actual[1,10])))+
  geom_segment(aes(x = 3.8, y = mean(Dw_actual[1,15]), xend = 4.2, yend = mean(Dw_actual[1,15])))+
  geom_segment(aes(x = 4.8, y = mean(Dw_actual[1,18]), xend = 5.2, yend = mean(Dw_actual[1,18])))+
  geom_segment(aes(x = 5.8, y = mean(Dw_actual[1,22]), xend = 6.2, yend = mean(Dw_actual[1,22])))+
  geom_segment(aes(x = 6.8, y = mean(Dw_actual[1,23]), xend = 7.2, yend = mean(Dw_actual[1,23])))+
  geom_segment(aes(x = 7.8, y = mean(Dw_actual[1,27]), xend = 8.2, yend = mean(Dw_actual[1,27])))+
  geom_segment(aes(x = 8.8, y = mean(Dw_actual[1,28]), xend = 9.2, yend = mean(Dw_actual[1,28])))+
  geom_segment(aes(x = 9.8, y = mean(Dw_actual[1,30]), xend = 10.2, yend = mean(Dw_actual[1,30])))+
  theme(panel.background = element_rect(fill = "white", colour = "black"), axis.text=element_text(size=18), axis.title = element_text(size = 18))
